------------------------------------------
-- Setting up libraries
------------------------------------------

local AddonName = "iGuild"
iGuild = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0", "AceTimer-3.0");

local Addon = iGuild;
Addon.Broker = LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
	type = "data source",
	text = "Loading...",
	icon = "Interface\\Addons\\iGuild\\Fubar_GuildFu",

	OnClick = function(_, button)
		if( button == "LeftButton" ) then
			if( IsAltKeyDown() and CanGuildInvite() ) then
				if( StaticPopup_FindVisible("ADD_GUILDMEMBER") ) then
					StaticPopup_Hide("ADD_GUILDMEMBER");
				else
					StaticPopup_Show("ADD_GUILDMEMBER");
				end
			else
				ToggleGuildFrame(1);
			end
		elseif( button == "RightButton" ) then
			Addon:OpenOptions();
		end
	end,

	OnEnter = function(anchor)
		Addon:ShowTooltip(anchor);
	end,
});

local QTip = LibStub("LibQTip-1.0");
local Tourist = LibStub("LibTourist-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

------------------------------------------
-- Variables and colors
------------------------------------------

local Tooltip;
local RosterTimer;

local NULL = "";
local COLUMN_HEADER_ROW = 0;

local COLOR_GOLD = "|cfffed100";

local ClassColors = {};
for k, v in pairs(LOCALIZED_CLASS_NAMES_MALE) do
	local c = RAID_CLASS_COLORS[k];
	ClassColors[v] = ("|cff%02x%02x%02x"):format(c.r *255, c.g *255, c.b *255);
end
for k, v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do
	local c = RAID_CLASS_COLORS[k];
	ClassColors[v] = ("|cff%02x%02x%02x"):format(c.r *255, c.g *255, c.b *255);
end

------------------------------------------
-- Columns
------------------------------------------

Addon.Columns = {
	level = {
		Label = L["Level"],
		Brush = function(v)
			local c = GetQuestDifficultyColor(v.charLevel);
			return ("|cff%02x%02x%02x%s|r"):format(c.r *255, c.g *255, c.b *255, v.charLevel);
		end,
		Align = "RIGHT",
		ShowLabel = false,
	},
	name = {
		Label = L["Name"],
		Brush = function(v)
			local c = GetQuestDifficultyColor(v.charLevel);
			return ("%s%s%s|r"):format(ClassColors[v.charClass], v.charStatus or NULL, v.charName);
		end,
		Align = "LEFT",
		ShowLabel = true,
	},
	zone = {
		Label = L["Zone"],
		Brush = function(v)
			local r, g, b = Tourist:GetFactionColor(v.charZone);
			return ("|cff%02x%02x%02x%s|r"):format(r *255, g *255, b *255, v.charZone);
		end,
		Align = "CENTER",
		ShowLabel = true,
	},
	rank = {
		Label = L["Rank"],
		Brush = function(v)
			return ("%s%s|r"):format(COLOR_GOLD, v.guildRank);
		end,
		Align = "CENTER",
		ShowLabel = true,
	},
	note = {
		Label = L["Note"],
		Brush = function(v)
			return ("%s%s|r"):format(COLOR_GOLD, v.guildNote);
		end,
		Align = "LEFT",
		ShowLabel = true,
	},
	acmpoints = {
		Label = L["Points"],
		Brush = function(v)
			return ("%s%s|r"):format(COLOR_GOLD, v.acmPoints);
		end,
		Align = "CENTER",
		ShowLabel = true,
	},
};

------------------------------------------
-- OnInitialize
------------------------------------------

function Addon:OnInitialize()
	-- 
	
	self.GuildRoster = {};

	self:RegisterEvent("GUILD_MOTD", "EventHandler");
	self:RegisterEvent("PLAYER_GUILD_UPDATE", "EventHandler");
	
	if( IsInGuild() ) then
		self:RegisterEvent("GUILD_ROSTER_UPDATE", "EventHandler");
		GuildRoster();
		RosterTimer = self:ScheduleRepeatingTimer(GuildRoster, 15);
	else
		self.Broker.text = L["No guild"];
	end
end

------------------------------------------
-- EventHandler
------------------------------------------

function Addon:EventHandler(event)
	if( IsInGuild() ) then
		if( not RosterTimer ) then
			self:RegisterEvent("GUILD_ROSTER_UPDATE", "EventHandler");
			GuildRoster();
			RosterTimer = self:ScheduleRepeatingTimer(GuildRoster, 15);
		end
		
		local Total, TotalOnline = GetNumGuildMembers();
		self.Broker.text = ("%d/%d"):format(TotalOnline, Total);
		
		self:SetupGuildRoster();
	else
		if( RosterTimer ) then
			self:UnregisterEvent("GUILD_ROSTER_UPDATE");
			self:CancelTimer(RosterTimer);
		end
		
		self.GuildRoster = {};
		self.Broker.text = L["No guild"];
	end
	
	if( QTip:IsAcquired(self) ) then
		self:UpdateTooltip();
	end
end

------------------------------------------
-- SetupGuildRoster
------------------------------------------

function Addon:SetupGuildRoster()
	local roster = {};
	
	local _, TotalOnline = GetNumGuildMembers();
	
	for i = 1, TotalOnline do
		local charName, guildRank, _, charLevel, charClass, charZone, 
			guildNote, officerNote, isOnline, charStatus, _, acmPoints, acmRank, charMobile = GetGuildRosterInfo(i);
		
		if( isOnline ) then
			local vartable = {
				charName = charName,
				charLevel = charLevel,
				charClass = charClass,
				charZone = charZone,
				charStatus = charStatus or nil,
				charMobile = charMobile or false,
				acmPoints = acmPoints,
				acmRank = acmRank,
				guildRank = guildRank,
				guildNote = guildNote or nil,
				officerNote = officerNote or nil,
			};
			
			roster[i] = vartable;
		end
	end
	
	self.GuildRoster = roster;
end

------------------------------------------
-- OnClick callbacks
------------------------------------------

local function cbTooltipLineClick(_, name)
	SetItemRef(("player:%s"):format(name), ("|Hplayer:%s|h[%s]|h"):format(name, name), "LeftButton");
end

------------------------------------------
-- UpdateTooltip
------------------------------------------

function Addon:UpdateTooltip()
	Tooltip:Clear();
	
	local ColumnCount = table.getn(self.db.Display);
	Tooltip:SetColumnLayout(ColumnCount);
	
	for line = COLUMN_HEADER_ROW, table.getn(self.GuildRoster) do
		local member;

		for col = 1, ColumnCount do
			local column = self.db.Display[col];
			
			if( line == COLUMN_HEADER_ROW ) then
				if( col == 1 ) then
					Tooltip:AddHeader(" ");
				end
				
				local label = "";
				if( self.db.Column[column].ShowLabel ) then
					label = self.Columns[column].Label;
				end
				
				Tooltip:SetCell(line +1, col, label, nil, self.db.Column[column].Align);--self.Columns[column].Align);
			else
				member = self.GuildRoster[line];
				
				if( col == 1 ) then
					Tooltip:AddLine(" ");
				end
				Tooltip:SetCell(line +1, col, self.Columns[column].Brush(member), nil, self.db.Column[column].Align); --self.Columns[column].Align);
			end
		end
		
		if( member ) then
			Tooltip:SetLineScript(line +1, "OnMouseDown", cbTooltipLineClick, member.charName);
		end
	end
end

------------------------------------------
-- ShowTooltip
------------------------------------------

function Addon:ShowTooltip(anchor)
	Tooltip = QTip:Acquire(iGuild)
	Tooltip:SetAutoHideDelay(0.1, anchor)
	Tooltip:SmartAnchorTo(anchor)
	self:UpdateTooltip()
	Tooltip:Show()
end